@@ -12,13 +12,24 @@ class TwimlException(Exception):
 class Verb(object):
     """Twilio basic verb object.
     """
+    GET = "GET"
+    POST = "POST"
+    nestables = None
+
     def __init__(self, **kwargs):
         self.name = self.__class__.__name__
         self.body = None
-        self.nestables = None
         self.verbs = []
         self.attrs = {}
 
+        if kwargs.get("waitMethod", "GET") not in ["GET", "POST"]:
+            raise TwimlException("Invalid waitMethod parameter, "
+                                 "must be 'GET' or 'POST'")
+
+        if kwargs.get("method", "GET") not in ["GET", "POST"]:
+            raise TwimlException("Invalid method parameter, "
+                                 "must be 'GET' or 'POST'")
+
         for k, v in kwargs.items():
             if k == "sender":
                 k = "from"
@@ -71,7 +82,7 @@ class Verb(object):
 
     def append(self, verb):
         if not self.nestables or verb.name not in self.nestables:
-            raise TwimlException("%s is not nestable inside %s" % \
+            raise TwimlException("%s is not nestable inside %s" %
                 (verb.name, self.name))
         self.verbs.append(verb)
         return verb
@@ -79,22 +90,24 @@ class Verb(object):
 
 class Response(Verb):
     """Twilio response object."""
+    nestables = [
+        'Say',
+        'Play',
+        'Gather',
+        'Record',
+        'Dial',
+        'Redirect',
+        'Pause',
+        'Hangup',
+        'Reject',
+        'Sms',
+        'Enqueue',
+        'Leave'
+        ]
+
     def __init__(self, **kwargs):
         """Version: Twilio API version e.g. 2008-08-01 """
-
-        Verb.__init__(self, **kwargs)
-        self.nestables = [
-            'Say',
-            'Play',
-            'Gather',
-            'Record',
-            'Dial',
-            'Redirect',
-            'Pause',
-            'Hangup',
-            'Reject',
-            'Sms',
-            ]
+        super(Response, self).__init__(**kwargs)
 
     def say(self, text, **kwargs):
         """Return a newly created :class:`Say` verb, nested inside this
@@ -136,6 +149,16 @@ class Response(Verb):
         :class:`Response` """
         return self.append(Dial(number, **kwargs))
 
+    def enqueue(self, name, **kwargs):
+        """Return a newly created :class:`Enqueue` verb, nested inside this
+        :class:`Response` """
+        return self.append(Enqueue(name, **kwargs))
+
+    def leave(self, **kwargs):
+        """Return a newly created :class:`Leave` verb, nested inside this
+        :class:`Response` """
+        return self.append(Leave(**kwargs))
+
     def record(self, **kwargs):
         """Return a newly created :class:`Record` verb, nested inside this
         :class:`Response` """
@@ -203,11 +226,11 @@ class Say(Verb):
     FRENCH = 'fr'
     GERMAN = 'de'
 
-    def __init__(self, text, voice=None, language=None, loop=None, **kwargs):
-        Verb.__init__(self, voice=voice, language=language, loop=loop,
-            **kwargs)
+    def __init__(self, text, **kwargs):
+        super(Say, self).__init__(**kwargs)
         self.body = text
 
+
 class Play(Verb):
     """Play an audio file at a URL
 
@@ -218,8 +241,8 @@ class Play(Verb):
                  Specifying '0' will cause the the :class:`Say` verb to loop
                  until the call is hung up. Defaults to 1.
     """
-    def __init__(self, url, loop=None, **kwargs):
-        Verb.__init__(self, loop=loop, **kwargs)
+    def __init__(self, url, **kwargs):
+        super(Play, self).__init__(**kwargs)
         self.body = url
 
 
@@ -229,8 +252,6 @@ class Pause(Verb):
     :param length: specifies how many seconds Twilio will wait silently before
                    continuing on.
     """
-    def __init__(self, length=None, **kwargs):
-        Verb.__init__(self, length=length, **kwargs)
 
 
 class Redirect(Verb):
@@ -244,26 +265,21 @@ class Redirect(Verb):
     GET = 'GET'
     POST = 'POST'
 
-    def __init__(self, url="", method=None, **kwargs):
-        Verb.__init__(self, method=method, **kwargs)
-        if method and method not in [self.GET, self.POST]:
-            raise TwimlException( \
-                "Invalid method parameter, must be 'GET' or 'POST'")
+    def __init__(self, url="", **kwargs):
+        super(Redirect, self).__init__(**kwargs)
         self.body = url
 
 
 class Hangup(Verb):
     """Hangup the call
     """
-    def __init__(self, **kwargs):
-        Verb.__init__(self)
 
 
 class Reject(Verb):
     """Hangup the call
+
+    :param reason: not sure
     """
-    def __init__(self, reason=None, **kwargs):
-        Verb.__init__(self, reason=reason, **kwargs)
 
 
 class Gather(Verb):
@@ -277,17 +293,10 @@ class Gather(Verb):
     """
     GET = 'GET'
     POST = 'POST'
+    nestables = ['Say', 'Play', 'Pause']
 
-    def __init__(self, action=None, method=None, numDigits=None, timeout=None,
-        finishOnKey=None, **kwargs):
-
-        Verb.__init__(self, action=action, method=method,
-            numDigits=numDigits, timeout=timeout, finishOnKey=finishOnKey,
-            **kwargs)
-        if method and (method != self.GET and method != self.POST):
-            raise TwimlException( \
-                "Invalid method parameter, must be 'GET' or 'POST'")
-        self.nestables = ['Say', 'Play', 'Pause']
+    def __init__(self, **kwargs):
+        super(Gather, self).__init__(**kwargs)
 
     def say(self, text, **kwargs):
         return self.append(Say(text, **kwargs))
@@ -314,8 +323,8 @@ class Number(Verb):
     :param number: phone number to dial
     :param sendDigits: key to press after connecting to the number
     """
-    def __init__(self, number, sendDigits=None, **kwargs):
-        Verb.__init__(self, sendDigits=sendDigits, **kwargs)
+    def __init__(self, number, **kwargs):
+        super(Number, self).__init__(**kwargs)
         self.body = number
 
 
@@ -325,7 +334,7 @@ class Client(Verb):
     :param name: Client name to connect to
     """
     def __init__(self, name, **kwargs):
-        Verb.__init__(self, **kwargs)
+        super(Client, self).__init__(**kwargs)
         self.body = name
 
 
@@ -341,13 +350,8 @@ class Sms(Verb):
     GET = 'GET'
     POST = 'POST'
 
-    def __init__(self, msg, to=None, sender=None, method=None, action=None,
-        statusCallback=None, **kwargs):
-        Verb.__init__(self, action=action, method=method, to=to, sender=sender,
-            statusCallback=statusCallback, **kwargs)
-        if method and (method != self.GET and method != self.POST):
-            raise TwimlException( \
-                "Invalid method parameter, must be GET or POST")
+    def __init__(self, msg, **kwargs):
+        super(Sms, self).__init__(**kwargs)
         self.body = msg
 
 
@@ -365,16 +369,8 @@ class Conference(Verb):
     GET = 'GET'
     POST = 'POST'
 
-    def __init__(self, name, muted=None, beep=None,
-        startConferenceOnEnter=None, endConferenceOnExit=None, waitUrl=None,
-        waitMethod=None, maxParticipants=None, **kwargs):
-        Verb.__init__(self, muted=muted, beep=beep,
-            startConferenceOnEnter=startConferenceOnEnter,
-            endConferenceOnExit=endConferenceOnExit, waitUrl=waitUrl,
-            waitMethod=waitMethod, maxParticipants=maxParticipants, **kwargs)
-        if waitMethod and (waitMethod != self.GET and waitMethod != self.POST):
-            raise TwimlException( \
-                "Invalid waitMethod parameter, must be GET or POST")
+    def __init__(self, name, **kwargs):
+        super(Conference, self).__init__(**kwargs)
         self.body = name
 
 
@@ -383,21 +379,25 @@ class Dial(Verb):
 
     :param action: submit the result of the dial to this URL
     :param method: submit to 'action' url using GET or POST
+    :param int timeout: The number of seconds to waits for the called
+                         party to answer the call
+    :param bool hangupOnStar: Allow the calling party to hang up on the
+                              called party by pressing the '*' key
+    :param int timeLimit: The maximum duration of the Call in seconds
+    :param callerId: The caller ID that will appear to the called party
+    :param bool record: Record both legs of a call within this <Dial>
     """
     GET = 'GET'
     POST = 'POST'
+    nestables = ['Number', 'Conference', 'Client', 'Queue']
 
-    def __init__(self, number=None, action=None, method=None, **kwargs):
-        Verb.__init__(self, action=action, method=method, **kwargs)
-        self.nestables = ['Number', 'Conference', 'Client']
+    def __init__(self, number=None, **kwargs):
+        super(Dial, self).__init__(**kwargs)
         if number and len(number.split(',')) > 1:
             for n in number.split(','):
                 self.append(Number(n.strip()))
         else:
             self.body = number
-        if method and (method != self.GET and method != self.POST):
-            raise TwimlException( \
-                "Invalid method parameter, must be GET or POST")
 
     def client(self, name, **kwargs):
         return self.append(Client(name, **kwargs))
@@ -408,6 +408,9 @@ class Dial(Verb):
     def conference(self, name, **kwargs):
         return self.append(Conference(name, **kwargs))
 
+    def queue(self, name, **kwargs):
+        return self.append(Queue(name, **kwargs))
+
     def addNumber(self, *args, **kwargs):
         return self.number(*args, **kwargs)
 
@@ -415,6 +418,49 @@ class Dial(Verb):
         return self.conference(*args, **kwargs)
 
 
+class Queue(Verb):
+    """Specify queue in a nested Dial element.
+
+    :param name: friendly name for the queue
+    :param url: url to a twiml document that executes after a call is dequeued
+                and before the call is connected
+    :param method: HTTP method for url GET/POST
+    """
+    GET = 'GET'
+    POST = 'POST'
+
+    def __init__(self, name, **kwargs):
+        super(Queue, self).__init__(**kwargs)
+        self.body = name
+
+
+class Enqueue(Verb):
+    """Enqueue the call into a specific queue.
+
+    :param name: friendly name for the queue
+    :param action: url to a twiml document that executes when the call
+                   leaves the queue. When dequeued via a <Dial> verb,
+                   this url is executed after the bridged parties disconnect
+    :param method: HTTP method for action GET/POST
+    :param wait_url: url to a twiml document that executes
+                     while the call is on the queue
+    :param wait_url_method: HTTP method for wait_url GET/POST
+    """
+    GET = 'GET'
+    POST = 'POST'
+
+    def __init__(self, name, **kwargs):
+        super(Enqueue, self).__init__(**kwargs)
+        self.body = name
+
+
+class Leave(Verb):
+    """Signals the call to leave its queue
+    """
+    GET = 'GET'
+    POST = 'POST'
+
+
 class Record(Verb):
     """Record audio from caller
 
@@ -425,14 +471,3 @@ class Record(Verb):
     """
     GET = 'GET'
     POST = 'POST'
-
-    def __init__(self, action=None, method=None, timeout=None,
-        finishOnKey=None, maxLength=None, transcribe=None,
-        transcribeCallback=None, playBeep=None, **kwargs):
-        Verb.__init__(self, action=action, method=method, timeout=timeout,
-                finishOnKey=finishOnKey, maxLength=maxLength,
-                transcribe=transcribe, transcribeCallback=transcribeCallback,
-                playBeep=playBeep, **kwargs)
-        if method and (method != self.GET and method != self.POST):
-            raise TwimlException( \
-                "Invalid method parameter, must be GET or POST")
